// $Id: event_handler.cc,v 1.12 2003/03/11 17:09:37 gordon Exp $	
//
// Copyright (C) 2002 Gordon Fraser <gordon@debian.org>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// See the COPYING file for more details
//

#include "event_handler.h"

//--------------------------------------------------------------------
EventHandler::EventHandler(DockApp* dockapp, Mixer* mixer, unsigned scroll)
  : dockapp_(dockapp), mixer_(mixer), current_channel_(0), num_channels_(mixer->getNumChannels()),
    wheel_scroll_(scroll), repeat_timer_(0), dragging_(false), button_left_pressed_(false), 
    button_right_pressed_(false), button_mute_pressed_(false), button_rec_pressed_(false)
{
  for(int counter = 0; counter < num_channels_; counter++)
  {
    if(mixer_->isSupported(counter) &&
       mixer_->isEnabled(counter))
    {
      current_channel_ = counter;
      break;
    }
  }
}

//--------------------------------------------------------------------
EventHandler::~EventHandler()
{
}

//--------------------------------------------------------------------
void EventHandler::loop()
{
  XEvent xev;  
  bool done = false;

  dockapp_->update(current_channel_,   
		   button_left_pressed_,
		   button_right_pressed_,
		   button_mute_pressed_,
		   button_rec_pressed_);
  mixer_->doUpdate(current_channel_);
  
  
  while(!done)
  {
    while(XPending(dockapp_->getDisplay())) 
    {
      XNextEvent(dockapp_->getDisplay(), &xev);
      int x = xev.xbutton.x-(dockapp_->getXHandler()->getWindowSize()/2-32);
      int y = xev.xbutton.y-(dockapp_->getXHandler()->getWindowSize()/2-32);

      switch(xev.type) 
      {
      case Expose:
	dockapp_->update(current_channel_,   
			 button_left_pressed_,
			 button_right_pressed_,
			 button_mute_pressed_,
			 button_rec_pressed_);
	break;
      case ButtonPress:
	handlePressEvent(x, y, xev.xbutton.button);
	break;
      case ButtonRelease:
	handleReleaseEvent();
	break;
      case MotionNotify:
	handleMotionEvent(x, y);
	break;
      case ClientMessage:
	if(xev.xclient.data.l[0] == (int)dockapp_->getXHandler()->getDeleteWin())
	  done=true;
	break;
      }
    }
            
    // keeping a button pressed causes scrolling throught the channels
    if(button_left_pressed_ || button_right_pressed_)
      handlePressedButton();

    if(mixer_->hasChanged())
    {
      mixer_->doUpdate(current_channel_);
      button_rec_pressed_  = mixer_->getCapture(current_channel_);
      button_mute_pressed_ = mixer_->isMuted(current_channel_);
      dockapp_->update(current_channel_,   
		       button_left_pressed_,
		       button_right_pressed_,
		       button_mute_pressed_,
		       button_rec_pressed_,
		       dragging_);
    }
      
    XFlush(dockapp_->getDisplay());
    usleep(100000);
  }

}

//--------------------------------------------------------------------
void EventHandler::handlePressEvent(int x, int y, int button_number)
{
  if(dockapp_->isLeftButton(x, y))
  {
    doLeftButton();
  }

  if(dockapp_->isRightButton(x, y))
  {
    doRightButton();
  }

  if(dockapp_->isRecButton(x, y))
  {
    doRecButton();
  }

  if(dockapp_->isMuteButton(x, y))
  {
    doMuteButton();
  }

  if(dockapp_->isVolumeBar(x, y))
  {
    doVolumeBar(x,y, button_number);
  }
  
  dockapp_->update(current_channel_, button_left_pressed_,
 		   button_right_pressed_, button_mute_pressed_,
 		   button_rec_pressed_, dragging_);
}

//--------------------------------------------------------------------
void EventHandler::handleReleaseEvent()
{
  dragging_             = false;
  button_left_pressed_  = false;
  button_right_pressed_ = false;

  dockapp_->update(current_channel_, button_left_pressed_,
		   button_right_pressed_, button_mute_pressed_,
		   button_rec_pressed_);
}

//--------------------------------------------------------------------
void EventHandler::handleMotionEvent(int x, int y)
{
  if(dockapp_->isVolumeBar(x, y) && dragging_)
  {
    // FIXXME
    //      int new_volume = ((60-y)*100)/(2*25);
    int new_volume = dockapp_->normalize(x, y);

    if(new_volume < 0)
      new_volume=0;

    if(dockapp_->isLeftVolumeBar(x,y))
      mixer_->setChannelLeft (current_channel_, new_volume);

    if(dockapp_->isRightVolumeBar(x,y))
      mixer_->setChannelRight(current_channel_, new_volume);

    dragging_ = new_volume;
  }
}

//--------------------------------------------------------------------
void EventHandler::handlePressedButton()
{
  repeat_timer_++;

  if(repeat_timer_ >= RPTINTERVAL)
  {
    if(button_left_pressed_)
    {
      doLeftButton();
    }
    else
    {
      doRightButton();
    }
      
    repeat_timer_ = 0;

    dockapp_->update(current_channel_,   
		     button_left_pressed_,
		     button_right_pressed_,
		     button_mute_pressed_,
		     button_rec_pressed_,
		     dragging_);
  }
}

//--------------------------------------------------------------------
void EventHandler::doLeftButton()
{
  int last_channel = current_channel_;

  do
  {
    if(current_channel_ < 1)
      current_channel_ = num_channels_-1;
    else
      current_channel_--;

    if(current_channel_ == last_channel)
      return;
  } 
  while(current_channel_ != last_channel && 
	(!mixer_->isSupported(current_channel_) ||
	 !mixer_->isEnabled(current_channel_)));
  
  button_left_pressed_ = true;
  repeat_timer_ = 0;

  mixer_->doUpdate();
}

//--------------------------------------------------------------------
void EventHandler::doRightButton()
{
  int last_channel = current_channel_;

  do
  {
    current_channel_++;
    if(current_channel_ >= num_channels_)
      current_channel_ = 0;

    if(current_channel_ == last_channel)
      return;
  } 
  while(current_channel_ != last_channel && (
					     !mixer_->isSupported(current_channel_) ||
					     !mixer_->isEnabled(current_channel_)));
  
  button_right_pressed_ = true;
  repeat_timer_ = 0;

  mixer_->doUpdate();    
}

//--------------------------------------------------------------------
void EventHandler::doMuteButton()
{
  if(mixer_->isMuted(current_channel_))
  {
    button_mute_pressed_ = false;
    mixer_->unMuteChannel(current_channel_);
  }
  else
  {
    button_mute_pressed_ = true;
    mixer_->muteChannel(current_channel_);
  }
}

//--------------------------------------------------------------------
void EventHandler::doRecButton()
{
  //   button_rec_pressed_ = true;
  button_rec_pressed_ = mixer_->toggleChannelRecord(current_channel_);
  //      forced_update = false;
}

//--------------------------------------------------------------------
void EventHandler::doVolumeBar(unsigned x, unsigned y, int button_number)
{
  int new_left = 0, new_right = 0;

  if(button_number < 4) 
  {
    // FIXXME
    //      new_left = ((60-y)*100)/(2*25);
    new_left = dockapp_->normalize(x, y);
    new_right = new_left;
    dragging_ = new_left;
  } 
  else if(button_number == 4) 
  {
    new_right = mixer_->getChannelRight(current_channel_)  + wheel_scroll_;
    new_left  = mixer_->getChannelLeft (current_channel_)  + wheel_scroll_;
	  
  } 
  else if(button_number == 5) 
  {
    new_right = mixer_->getChannelRight(current_channel_)  - wheel_scroll_;
    new_left  = mixer_->getChannelLeft (current_channel_)  - wheel_scroll_;
  }

  if(new_left <= 0)
    new_left = 0;

  if(new_right <= 0)
    new_right = 0;

  if(dockapp_->isLeftVolumeBar(x,y))
  {
    mixer_->setChannelLeft(current_channel_, new_left);
    dragging_ = new_left;
  }

  if(dockapp_->isRightVolumeBar(x,y))
  {
    mixer_->setChannelRight(current_channel_, new_right);
    dragging_ = new_right;
  }
}

